package com.hive.sample


import android.app.Activity
import android.app.AlertDialog
import android.app.Dialog
import android.content.DialogInterface
import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.View
import android.widget.*
import androidx.annotation.StringRes
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.gcp.hivecore.Configuration.HIVELanguage
import com.gcp.hivecore.Configuration.ZoneType
import com.gcp.hivecore.Logger
import com.hive.Configuration
import com.hive.Const
import com.hive.base.Android
import com.hive.base.Property
import com.hive.sample.uihelper.ViewPagerBase
import com.hive.ui.Resource
import java.util.*


/**
 * HIVE SDK 의 Configuration 설정 대화 상자
 *
 * @author ryuvsken
 */
class ConfigurationDialog(private val activity: Activity) : Dialog(activity, android.R.style.Theme_Material_Light) {

    /**
     * Configuration Data
     *
     * @param appId                     // 입력된 AppId
     * @param hiveCertificationKey      // Hive 콘솔에서 발급 받은 AppKey
     * @param zoneType                  // 선택된 HIVE 플랫폼 서비스 형태
     * @param hiveTheme                 // 선택된 HIVE 테마
     * @param hiveOrientation           // 선택된 Hive Orientation
     * @param gameLanguage              // 선택된 게임 언어
     * @param gameServerId              // 선택된 게임 서버
     * @param isLogging                 // HIVE SDK 동작 로그 여부
     * @param logFilter                 // HIVE SDK 로그 필터 타입
     * @param loggingUserId             // 리모트 로그를 위한 사용자 ID
     * @param isRemoteLogging           // 리모트 로그 사용 유무
     * @param isRemoteDetailLogging     // 리모트 상세 로그 사용 유무
     * @param isAgeGateU13              // AgeGateU13 사용 유무
     * @param hiveCommunityUrl          // Hive 커뮤니티 URL
     */
    data class ConfigurationData(
            var appId: String = "",
            var hiveCertificationKey: String = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJIaXZlIiwiaWF0IjoxNjAyMDU2NzI2LCJqdGkiOiIxODczMTExMzIwIn0.3soFiHTPlObCoqR5xX9ZeOQTSvnHrHDHWmopP3QfWtY",
            var zoneType: ZoneType = ZoneType.SANDBOX,
            var hiveTheme: com.hive.ui.Configuration.HiveTheme = com.hive.ui.Configuration.HiveTheme.hiveLight,
            var hiveOrientation: String = "UNDEFINED",
            var gameLanguage: String = "en",
            var gameServerId: String = "kr",
            var isLogging: Boolean = false,
            var loggingUserId: String? = null,
            var logFilter: Logger.LogFilter = Logger.LogFilter(),
            var isRemoteLogging: Boolean = false,
            var isRemoteDetailLogging: Boolean = false,
            var isAgeGateU13: Boolean = false,
            var isOnHive: Boolean = true,
            var hiveCommunityUrl: String = "",
    )


    object PropertyKeys {
        const val appId                 = "sample.appId"
        const val hiveCertificationKey  = "sample.hiveCertificationKey"
        const val zoneType              = "sample.zoneType"
        const val hiveTheme             = "sample.hiveTheme"
        const val hiveOrientation       = "sample.hiveOrientation"
        const val gameLanguage          = "sample.gameLanguage"
        const val gameServerId          = "sample.gameServerId"
        const val isLogging             = "sample.isLogging"
        const val loggingUserId         = "sample.loggingUserId"
        const val coreLogFilter         = "sample.coreLogFilter"
        const val serviceLogFilter      = "sample.serviceLogFilter"
        const val isRemoteLogging       = "sample.isRemoteLogging"
        const val isRemoteDetailLogging = "sample.isRemoteDetailLogging"
        const val isAgeGateU13          = "sample.isAgeGateU13"
        const val isOnHive              = "sample.isOnHive"
        const val hiveCommunityUrl      = "sample.hiveCommunityUrl"
    }


    var data: ConfigurationData = ConfigurationData()

    private val property: Property = Property("sample.prop").apply {
        this.isPrefixFileName = false
    }

    private val hiveOrientationList = ArrayList<String>()
    private val languageMap = LinkedHashMap<String, LanguageDisplayContent>()


    init {
        languageMap["en"] = LanguageDisplayContent("en", Locale.ENGLISH, R.string.hive_language_en)
        languageMap["ko"] = LanguageDisplayContent("ko", Locale.KOREAN, R.string.hive_language_ko)
        languageMap["zh-Hans"] = LanguageDisplayContent("zh-Hans", Locale.SIMPLIFIED_CHINESE, R.string.hive_language_zh_Hans)
        languageMap["zh-Hant"] = LanguageDisplayContent("zh-Hant", Locale.TRADITIONAL_CHINESE, R.string.hive_language_zh_Hant)
        languageMap["ja"] = LanguageDisplayContent("ja", Locale.JAPANESE, R.string.hive_language_ja)
        languageMap["ru"] = LanguageDisplayContent("ru", Locale("ru"), R.string.hive_language_ru)
        languageMap["fr"] = LanguageDisplayContent("fr", Locale.FRENCH, R.string.hive_language_fr)
        languageMap["de"] = LanguageDisplayContent("de", Locale.GERMAN, R.string.hive_language_de)
        languageMap["es"] = LanguageDisplayContent("es", Locale("es"), R.string.hive_language_es)
        languageMap["pt"] = LanguageDisplayContent("pt", Locale("pt"), R.string.hive_language_pt)
        languageMap["id"] = LanguageDisplayContent("id", Locale("id"), R.string.hive_language_id)
//		languageMap["ms"] = LanguageDisplayContent("ms", Locale("ms"), R.string.hive_language_ms));// ms_MY
        languageMap["th"] = LanguageDisplayContent("th", Locale("th"), R.string.hive_language_th)
        languageMap["vi"] = LanguageDisplayContent("vi", Locale("vi"), R.string.hive_language_vi)// vi_VN
        languageMap["it"] = LanguageDisplayContent("it", Locale.ITALIAN, R.string.hive_language_it)
        languageMap["tr"] = LanguageDisplayContent("tr", Locale("tr"), R.string.hive_language_tr)
        languageMap["ar"] = LanguageDisplayContent("ar", Locale("ar"), R.string.hive_language_ar)

        hiveOrientationList.add("UNDEFINED")
        hiveOrientationList.add("ALL")
        hiveOrientationList.add("PORTRAIT")
        hiveOrientationList.add("LANDSCAPE")

    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.test_view_config)

        buildUI()
    }

    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)

        if(hasFocus){
            com.hive.ui.Util.setOnSystemUiVisibilityChangeListener(window?.decorView)
        }
    }

    // 출력 view에 값을 저장하기 위한 폴링 함수.
    // 광고 아이디는 구글 서버로부터 asynchronous 하게 수신되므로,
    // UI thread 에 handler를 이용하여 post 한다.
    private fun updateAdvertisingIdTextView() {

        if (!Android.isGooglePlayServicesAvailable(activity, false)) {
            return
        }

        val handler = Handler()

        Thread(Runnable {
            // TODO Auto-generated method stub
            do {
                if (com.hive.base.Android.advertisingId == null) {
                    try {
                        Thread.sleep(500)
                    } catch (e: Exception) {
                    }

                } else {
                    handler.post {
                        val txtView = this@ConfigurationDialog.findViewById<View>(R.id.txtADID) as TextView
                        txtView.text = com.hive.base.Android.advertisingId
                    }
                    break
                }
            } while (true)
        }).start()
    }

    private fun buildUI() {

        // hive_config.xml 에 설정되어 있는 값을 읽어온다.
        val `is` = Resource.openRawResource(activity, "hive_config")
        if (`is` == null) {
            Log.e(Const.TAG, "hive_config.xml file not found!")
        }

        val root = Resource.readXmlToJSON(`is`!!)
        if (root == null) {
            Log.e(Const.TAG, "hive_config.xml file is invalid format!")
        }


        var defAppId = Android.getPackageName() // default로 AndroidManifest.xml 의 packageName 을 설정
        var defZoneType = ZoneType.REAL
        var defUseLog: Boolean = true
        var defUseAgeGateU13: Boolean? = false
        try {

            val properties = root!!.getJSONObject("properties")
            defAppId = properties.optString("appId", defAppId)
            defZoneType = ZoneType.valueOf(properties.optString("zone", ZoneType.SANDBOX.name).toUpperCase(Locale.US))
            defUseLog = properties.optBoolean("useLog", true)
            defUseAgeGateU13 = properties.optBoolean("ageGateU13", false)
        } catch (e: Exception) {
        }


        var txtView: TextView
        var edtText: EditText
        var btnToggle: ToggleButton
        var switchBtn: Switch

        val spAdapter: ArrayAdapter<CharSequence>
        val spThemeAdapter: ArrayAdapter<com.hive.ui.Configuration.HiveTheme>
        val spOrientationAdapter: ArrayAdapter<String>
        val spLanguageAdapter: ArrayAdapter<LanguageDisplayContent>
        val spCoreLogFilterAdapter: ArrayAdapter<Logger.LogType>
        val spServiceLogFilterAdapter: ArrayAdapter<Logger.LogType>
        var spinner: Spinner


        // SDK Version 출력
        val sb = StringBuffer()
        sb.append("HIVE SDK=")
        sb.append(com.hive.Configuration.hiveSDKVersion)

        val refSDKVersion = com.hive.Configuration.referenceSDKVersion
        if (refSDKVersion.length > 0) {
            sb.append("\n")
            sb.append(refSDKVersion)
        }
        txtView = this.findViewById<View>(R.id.txtSDKVer) as TextView
        txtView.text = sb.toString()

        // 광고 아이디 출력용 view 저장.
        txtView = this.findViewById<View>(R.id.txtADID) as TextView
        txtView.text = "wait..." // 초기값.
        updateAdvertisingIdTextView() // 값의 업데이트를 위한 폴링 함수 호

        // AppId 설정
        data.appId = property.getValue(PropertyKeys.appId, defAppId!!)
        edtText = this.findViewById<View>(R.id.edtPackageInfo) as EditText
        edtText.setText(data.appId)

        // AppKey 설정
        data.hiveCertificationKey = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJIaXZlIiwiaWF0IjoxNjAyMDU2NzI2LCJqdGkiOiIxODczMTExMzIwIn0.3soFiHTPlObCoqR5xX9ZeOQTSvnHrHDHWmopP3QfWtY";
        edtText = this.findViewById<View>(R.id.edtHiveCertificationKey) as EditText
        edtText.setText(data.hiveCertificationKey)

        // Hive 플랫폼의 서버 선택. sandbox 는 개발용, real 은 실계
        spAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spAdapter.add("SANDBOX")
        spAdapter.add("REAL")
        spAdapter.add("TEST")
        spAdapter.add("DEV")

        spinner = this.findViewById<View>(R.id.spZoneType) as Spinner
        spinner.adapter = spAdapter

        var value = property.getValue(PropertyKeys.zoneType, defZoneType.name)
        if ("REAL" == value) {
            spinner.setSelection(1)
        } else if ("TEST" == value) {
            spinner.setSelection(2)
        }

        // Hive Theme 선택
        spThemeAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spThemeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spThemeAdapter.add(com.hive.ui.Configuration.HiveTheme.hiveLight)
        spThemeAdapter.add(com.hive.ui.Configuration.HiveTheme.hiveDark)

        spinner = this.findViewById<View>(R.id.spThemeType) as Spinner
        spinner.adapter = spThemeAdapter

        var themeValue = property.getValue(PropertyKeys.hiveTheme, com.hive.ui.Configuration.HiveTheme.hiveLight.name)
        if ("hiveLight" == themeValue) {
            spinner.setSelection(0)
        } else if ("hiveDark" == themeValue) {
            spinner.setSelection(1)
        }



        spOrientationAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spOrientationAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        for (value in hiveOrientationList) {
            spOrientationAdapter.add(value)
        }

        spinner = this.findViewById<View>(R.id.spHiveOrientationType) as Spinner
        spinner.adapter = spOrientationAdapter

        value = property.getValue(PropertyKeys.hiveOrientation, "UNDEFINED")
        spinner.setSelection(spOrientationAdapter.getPosition(value))


        // HiveSDK 의 사용 가능 언어 선택
        spLanguageAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        for (content in languageMap.values) {
            spLanguageAdapter.add(content)
        }

        spinner = this.findViewById<View>(R.id.spHiveLanguage) as Spinner
        spinner.adapter = spLanguageAdapter

        value = property.getValue(PropertyKeys.gameLanguage, "en")
        spinner.setSelection(spLanguageAdapter.getPosition(languageMap[value]))


        // HiveSDK 의 사용 게임 서버 선택
        value = property.getValue(PropertyKeys.gameServerId, "kr")
        edtText = this.findViewById<View>(R.id.edtHiveGameServerId) as EditText
        edtText.setText(value)


        // HIVE SDK 동작 로그 여부
        value = property.getValue(PropertyKeys.isLogging, if (defUseLog) "true" else "false")
        btnToggle = this.findViewById<View>(R.id.tbUseLog) as ToggleButton
        btnToggle.isChecked = "true" == value


        // Hive SDK coreLogFilter 설정 값
        spCoreLogFilterAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spCoreLogFilterAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        Logger.LogType.values().forEach {
            spCoreLogFilterAdapter.add(it)
        }

        spinner = this.findViewById<View>(R.id.spCoreLogFilter) as Spinner
        spinner.adapter = spCoreLogFilterAdapter

        value = property.getValue(PropertyKeys.coreLogFilter, "Verbose")
        val coreLogType = Logger.LogType.valueOf(value)
        spinner.setSelection(spCoreLogFilterAdapter.getPosition(coreLogType))


        // Hive SDK serviceLogFilter 설정 값
        spServiceLogFilterAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item)
        spServiceLogFilterAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

        Logger.LogType.values().forEach {
            spServiceLogFilterAdapter.add(it)
        }

        spinner = this.findViewById<View>(R.id.spServiceLogFilter) as Spinner
        spinner.adapter = spServiceLogFilterAdapter

        value = property.getValue(PropertyKeys.serviceLogFilter, "Verbose")
        val serviceLogType = Logger.LogType.valueOf(value)
        spinner.setSelection(spServiceLogFilterAdapter.getPosition(serviceLogType))


        // 리모트 로그를 위한 사용자 ID
        value = property.getValue(PropertyKeys.loggingUserId, "")
        edtText = this.findViewById<View>(R.id.edtLogUserId) as EditText
        edtText.setText(value)

        // 리모트 로그 사용 유무
        value = property.getValue(PropertyKeys.isRemoteLogging, "false")
        btnToggle = this.findViewById<View>(R.id.tbRemoteLog) as ToggleButton
        btnToggle.isChecked = "true" == value

        // 리모트 상세 로그 사용 유무
        value = property.getValue(PropertyKeys.isRemoteDetailLogging, "false")
        btnToggle = this.findViewById<View>(R.id.tbRemoteDetailLog) as ToggleButton
        btnToggle.isChecked = "true" == value

        // AgeGateU13 사용 유무
        value = property.getValue(PropertyKeys.isAgeGateU13, "false")
        btnToggle = this.findViewById<View>(R.id.tbAgeGateU13) as ToggleButton
        btnToggle.isChecked = "true" == value

        // Hive 커뮤니티 URL
        value = property.getValue(PropertyKeys.hiveCommunityUrl, "")
        edtText = this.findViewById<View>(R.id.edtHiveCommunityUrl) as EditText
        edtText.setText(value)

        // getPermissionViewData에 사용할 Resource Language 등록
        spinner = this.findViewById(R.id.spinnerResourceLanguages) as Spinner
        var resourceLanguages = arrayOf("de", "en", "es", "fr", "in", "it", "ja", "ko", "pt", "ru","th", "tr", "vi", "zh-hans", "zh-hant", "ar" )
        var adapter = ArrayAdapter<String>(activity, android.R.layout.simple_spinner_item, resourceLanguages)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        spinner.setAdapter(adapter)

        // HIVE SDK 권한고지 팝업 노출 여부 설정
        value = property.getValue(PropertyKeys.isOnHive, "true")
        switchBtn = this.findViewById<View>(R.id.switchSetHivePermissionViewOn) as Switch
        switchBtn.isChecked = "true" == value


        // HiveSDK 설정 파일 (hive_config.xml) 보기
        this.findViewById<View>(R.id.btnViewConfigurationFile).setOnClickListener {
            val xmlContent = Resource.readFile(Resource.openRawResource(activity, "hive_config")) ?: ""
            ViewPagerBase.showLogDialog(activity, xmlContent)
        }


        // 설정 초기화
        this.findViewById<View>(R.id.btnResetConfiguration).setOnClickListener {
            val property = this@ConfigurationDialog.property
            property.deletePropertyFile()

            this@ConfigurationDialog.buildUI()
        }


        // 설정 적용
        this.findViewById<View>(R.id.btnApplyConfiguration).setOnClickListener { this@ConfigurationDialog.dismiss() }

        // HIVE SDK 권한고지 팝업 노출 여부 설정값 적용
        this.findViewById<View>(R.id.btnSetHivePermissionViewOn).setOnClickListener {
            Configuration.setHivePermissionViewOn(switchBtn.isChecked)
        }

        //커스텀 권한고지를 위한 데이터
        this.findViewById<View>(R.id.btnGetPermissionViewData).setOnClickListener {
            var languageStr = spinner.selectedItem.toString()
            var language = HIVELanguage.LanguageEN

            for (item in HIVELanguage.values()) {
                if (item.value == languageStr) {
                    language = item
                    break
                }
                //UI에서는 "id"값으로 가져오므로 Android에서 따로 "in"으로 로컬라이징 해야함.
                if (languageStr == "id") {
                    language = com.gcp.hivecore.Configuration.HIVELanguage.LanguageIN
                    break
                }
            }
            var data = Configuration.getPermissionViewData(language)

            var popupTitle = data.contents
            var permissionViewUnitDatas = data.permissions
            var sb = StringBuilder()

            for (permissionViewUnitData in permissionViewUnitDatas) {
                sb.append("Title : ").append(permissionViewUnitData.title).append("\n")
                sb.append("Description : ").append(permissionViewUnitData.contents).append("\n")
                sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n")
            }

            var commonDatas = data.commons;
            for (commonUnitData in commonDatas) {
                sb.append("Title : ").append(commonUnitData.title).append("\n")
                sb.append("Description : ").append(commonUnitData.contents).append("\n")
                sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n")
            }

            var builder = AlertDialog.Builder(activity)
            builder.setTitle(popupTitle)
            builder.setMessage(sb.toString())
            builder.setPositiveButton("OK", DialogInterface.OnClickListener { dialog, which ->  })
            builder.create().show()
        }
    }


    fun showConfigurationDialog(listener: ConfigurationDialogListener?) {

        this.setOnDismissListener {
            var spinner: Spinner

            with(data) {

                appId = findViewById<EditText>(R.id.edtPackageInfo).text.toString()
                property.setValue(PropertyKeys.appId, appId)

                hiveCertificationKey = findViewById<EditText>(R.id.edtHiveCertificationKey).text.toString()
                property.setValue(PropertyKeys.hiveCertificationKey, hiveCertificationKey)

                spinner = findViewById(R.id.spZoneType)
                val selectedValue = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                zoneType = ZoneType.valueOf(selectedValue)
                property.setValue(PropertyKeys.zoneType, zoneType.name.toUpperCase(Locale.US))

                spinner = findViewById(R.id.spThemeType)
                val selectedTheme = spinner.adapter.getItem(spinner.selectedItemPosition) as com.hive.ui.Configuration.HiveTheme
                hiveTheme = selectedTheme
                property.setValue(PropertyKeys.hiveTheme, hiveTheme.name)

                spinner = findViewById(R.id.spHiveOrientationType)
                val selectedOrientation = spinner.adapter.getItem(spinner.selectedItemPosition) as String
                hiveOrientation = selectedOrientation
                property.setValue(PropertyKeys.hiveOrientation, hiveOrientation)

                spinner = findViewById(R.id.spHiveLanguage)
                val content = spinner.adapter.getItem(spinner.selectedItemPosition) as LanguageDisplayContent
                gameLanguage = content.languageCode
                property.setValue(PropertyKeys.gameLanguage, gameLanguage)

                gameServerId = findViewById<EditText>(R.id.edtHiveGameServerId).text.toString()
                property.setValue(PropertyKeys.gameServerId, gameServerId)

                isLogging = findViewById<ToggleButton>(R.id.tbUseLog).isChecked
                property.setValue(PropertyKeys.isLogging, if (isLogging) "true" else "false")

                loggingUserId = findViewById<EditText>(R.id.edtLogUserId).text.toString()
                property.setValue(PropertyKeys.loggingUserId, loggingUserId)

                spinner = findViewById(R.id.spCoreLogFilter)
                val coreLogType = spinner.adapter.getItem(spinner.selectedItemPosition) as Logger.LogType
                logFilter.coreLog = coreLogType
                property.setValue(PropertyKeys.coreLogFilter, coreLogType.name)

                spinner = findViewById(R.id.spServiceLogFilter)
                val serviceLogType = spinner.adapter.getItem(spinner.selectedItemPosition) as Logger.LogType
                logFilter.serviceLog = serviceLogType
                property.setValue(PropertyKeys.serviceLogFilter, serviceLogType.name)

                isRemoteLogging = findViewById<ToggleButton>(R.id.tbRemoteLog).isChecked
                property.setValue(PropertyKeys.isRemoteLogging, if (isRemoteLogging) "true" else "false")

                isRemoteDetailLogging = findViewById<ToggleButton>(R.id.tbRemoteDetailLog).isChecked
                property.setValue(PropertyKeys.isRemoteDetailLogging, if (isRemoteDetailLogging) "true" else "false")

                isAgeGateU13 = findViewById<ToggleButton>(R.id.tbAgeGateU13).isChecked
                property.setValue(PropertyKeys.isAgeGateU13, if (isAgeGateU13) "true" else "false")

                isOnHive = findViewById<Switch>(R.id.switchSetHivePermissionViewOn).isChecked
                property.setValue(PropertyKeys.isOnHive, if (isOnHive) "true" else "false")

                hiveCommunityUrl = findViewById<EditText>(R.id.edtHiveCommunityUrl).text.toString()
                property.setValue(PropertyKeys.hiveCommunityUrl, hiveCommunityUrl)
            }


            property.writeProperties()

            listener?.onConfigurationSelect(data)
        }


        this.setCancelable(false)
        this.show()
    }


    private inner class LanguageDisplayContent internal constructor(languageCode: String, locale: Locale, @StringRes displayStringId: Int) {

        internal var languageCode = ""
        internal var locale: Locale? = null
        @StringRes
        internal var displayStringId = 0

        init {
            this.languageCode = languageCode
            this.locale = locale
            this.displayStringId = displayStringId
        }

        override fun toString(): String {

            return if (displayStringId != 0)
                activity.resources.getString(displayStringId) + "(" + languageCode + ")"
            else
                languageCode
        }
    }


    /**
     * HIVE SDK 설정이 완료될 경우 호출되는 리스너
     *
     * @author ryuvsken
     */
    interface ConfigurationDialogListener {

        fun onConfigurationSelect(data: ConfigurationData)
    }
}
